# encoding: utf-8
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.

module MsRest
  #
  # Class which represents an error meaning that invalid Model object was created by user or provided from server.
  #
  class ValidationError < RestError
  end
end
